==> acl2_8.0dfsg.orig.tar.gz: acl2-8.0dfsg/books/centaur/sv/mods/path-string.lisp <==
; SV - Symbolic Vector Hardware Analysis Framework
; Copyright (C) 2014-2015 Centaur Technology
;
; Contact:
;   Centaur Technology Formal Verification Group
;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
;   http://www.centtech.com/
;
; License: (An MIT/X11-style license)
;
;   Permission is hereby granted, free of charge, to any person obtaining a
;   copy of this software and associated documentation files (the "Software"),
;   to deal in the Software without restriction, including without limitation
;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
;   and/or sell copies of the Software, and to permit persons to whom the
;   Software is furnished to do so, subject to the following conditions:
;
;   The above copyright notice and this permission notice shall be included in
;   all copies or substantial portions of the Software.
;
;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
;   DEALINGS IN THE SOFTWARE.
;
; Original authors: Sol Swords <sswords@centtech.com>
;                   Jared Davis <jared@centtech.com>



(in-package "SV")

(include-book "address")
(include-book "std/strings/decimal" :dir :system)
(include-book "centaur/vl/util/printedlist" :dir :system)

(local (std::add-default-post-define-hook :fix))

(defmacro rlist* (&rest args)
  (xxxjoin 'cons (reverse args)))

(define name->string-under-path ((x name-p) (p vl-printedlist-p))
  :returns (p1 vl-printedlist-p)
  (let ((p (vl::vl-printedlist-fix p))
        (x (name-fix x)))
    (if (integerp x)
        (rlist* p "[" (if (< x 0) "-" "") (str::natstr (abs x)) "]")
      (if (stringp x)
          (rlist* p "." x)
        (rlist* p ".SELF")))))

(define name->string-base ((x name-p) (p vl-printedlist-p))
  :returns (p1 vl-printedlist-p)
  (let ((p (vl::vl-printedlist-fix p))
        (x (name-fix x)))
    (if (integerp x)
        (rlist* p "_" (if (< x 0) "-" "") (str::natstr (abs x)) "_")
      (if (stringp x)
          (rlist* p x)
        (rlist* p "SELF")))))

(define name->string ((x name-p))
  :returns (s stringp)
  (vl::vl-printedlist->string (name->string-base x nil)))

(define path->string-aux ((x path-p) (p vl-printedlist-p))
  :returns (p1 vl-printedlist-p)
  :measure (path-count x)
  :guard-hints ((and stable-under-simplificationp
                     '(:in-theory (enable name-p))))
  (path-case x
    :wire (name->string-under-path x.name p)
    :scope (b* ((p (name->string-under-path x.namespace p)))
             (path->string-aux x.subpath p))))

(define path->string ((x path-p) (p vl-printedlist-p))
  :returns (p1 vl-printedlist-p)
  (path-case x
    :wire (name->string-base x.name p)
    :scope (b* ((p (name->string-base x.namespace p)))
             (path->string-aux x.subpath p))))

(define path->string-top ((x path-p))
  :returns (str stringp)
  (vl::vl-printedlist->string (path->string x nil)))

==> acl2_8.3dfsg.orig.tar.gz: acl2-8.3dfsg/books/centaur/sv/mods/path-string.lisp <==
; SV - Symbolic Vector Hardware Analysis Framework
; Copyright (C) 2014-2015 Centaur Technology
;
; Contact:
;   Centaur Technology Formal Verification Group
;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
;   http://www.centtech.com/
;
; License: (An MIT/X11-style license)
;
;   Permission is hereby granted, free of charge, to any person obtaining a
;   copy of this software and associated documentation files (the "Software"),
;   to deal in the Software without restriction, including without limitation
;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
;   and/or sell copies of the Software, and to permit persons to whom the
;   Software is furnished to do so, subject to the following conditions:
;
;   The above copyright notice and this permission notice shall be included in
;   all copies or substantial portions of the Software.
;
;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
;   DEALINGS IN THE SOFTWARE.
;
; Original authors: Sol Swords <sswords@centtech.com>
;                   Jared Davis <jared@centtech.com>



(in-package "SV")

(include-book "address")
(include-book "std/strings/decimal" :dir :system)
(include-book "centaur/vl/util/printedlist" :dir :system)

(local (std::add-default-post-define-hook :fix))

(defmacro rlist* (&rest args)
  (xxxjoin 'cons (reverse args)))

(define name->string-under-path ((x name-p) (slashesp) (p vl-printedlist-p))
  :returns (p1 vl-printedlist-p)
  (let ((p (vl::vl-printedlist-fix p))
        (x (name-fix x)))
    (if (integerp x)
        (rlist* p "[" (if (< x 0) "-" "") (str::natstr (abs x)) "]")
      (if (stringp x)
          (b* ((escape (or (position #\[ x)
                           (position #\] x)))
               (escx (if escape (cat "\\" x) x)))
            (rlist* p (if slashesp "/" ".") escx))
        (rlist* p (if slashesp "/SELF" ".SELF"))))))

(define name->string-base ((x name-p) (p vl-printedlist-p))
  :returns (p1 vl-printedlist-p)
  (let ((p (vl::vl-printedlist-fix p))
        (x (name-fix x)))
    (if (integerp x)
        (rlist* p "_" (if (< x 0) "-" "") (str::natstr (abs x)) "_")
      (if (stringp x)
          (b* ((escape (or (position #\[ x)
                           (position #\] x)))
               (escx (if escape (cat "\\" x) x)))
            (rlist* p escx))
        (rlist* p "SELF")))))

(define name->string ((x name-p))
  :returns (s stringp)
  (vl::vl-printedlist->string (name->string-base x nil)))

(define path->string-aux ((x path-p) (slashesp) (p vl-printedlist-p))
  :returns (p1 vl-printedlist-p)
  :measure (path-count x)
  :guard-hints ((and stable-under-simplificationp
                     '(:in-theory (enable name-p))))
  (path-case x
    :wire (name->string-under-path x.name slashesp p)
    :scope (b* ((p (name->string-under-path x.namespace slashesp p)))
             (path->string-aux x.subpath slashesp p))))

(define path->string ((x path-p) (p vl-printedlist-p))
  :returns (p1 vl-printedlist-p)
  (path-case x
    :wire (name->string-base x.name p)
    :scope (b* ((p (name->string-base x.namespace p)))
             (path->string-aux x.subpath nil p))))

(define path->string-top ((x path-p))
  :returns (str stringp)
  (vl::vl-printedlist->string (path->string x nil)))

(define path->string-slashed ((x path-p) (p vl-printedlist-p))
  :returns (p1 vl-printedlist-p)
  (path-case x
    :wire (name->string-base x.name p)
    :scope (b* ((p (name->string-base x.namespace p)))
             (path->string-aux x.subpath t p))))

(define path->string-slashed-top ((x path-p))
  :returns (str stringp)
  (vl::vl-printedlist->string (path->string-slashed x nil)))
